# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-25 16:26
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_projects', '0003_auto_20161025_1609'),
    ]

    operations = [
        migrations.CreateModel(
            name='Job',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start_date', models.DateTimeField(blank=True, null=True, verbose_name='Start date')),
                ('end_date', models.DateTimeField(blank=True, null=True, verbose_name='End date')),
                ('can_be_done_remotely', models.BooleanField(default=False, verbose_name='This job can be done remotely')),
            ],
            options={
                'verbose_name': 'Job',
                'verbose_name_plural': 'Jobs',
            },
        ),
        migrations.CreateModel(
            name='JobDate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=20, null=True, verbose_name='Start date')),
                ('start_date', models.DateTimeField(blank=True, null=True, verbose_name='Start date')),
                ('end_date', models.DateTimeField(blank=True, null=True, verbose_name='End date')),
            ],
        ),
        migrations.CreateModel(
            name='Work',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('weekly_hours', models.PositiveSmallIntegerField(blank=True, null=True, verbose_name='Weekly hours')),
                ('description', models.CharField(blank=True, max_length=4000, null=True, verbose_name='Description')),
                ('can_be_done_remotely', models.BooleanField(default=False, verbose_name='This work can be done remotely.')),
                ('project', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='ovp_projects.Project')),
            ],
            options={
                'verbose_name': 'work',
                'verbose_name_plural': 'works',
            },
        ),
        migrations.AlterModelOptions(
            name='role',
            options={'verbose_name': 'role', 'verbose_name_plural': 'roles'},
        ),
        migrations.AddField(
            model_name='role',
            name='details',
            field=models.TextField(blank=True, default=None, max_length=1024, null=True, verbose_name='Details'),
        ),
        migrations.AddField(
            model_name='role',
            name='name',
            field=models.CharField(blank=True, default=None, max_length=50, null=True, verbose_name='Role name'),
        ),
        migrations.AddField(
            model_name='role',
            name='prerequisites',
            field=models.TextField(blank=True, default=None, max_length=1024, null=True, verbose_name='Prerequisites'),
        ),
        migrations.AddField(
            model_name='role',
            name='vacancies',
            field=models.PositiveSmallIntegerField(blank=True, default=None, null=True, verbose_name='Vacancies'),
        ),
        migrations.AddField(
            model_name='job',
            name='dates',
            field=models.ManyToManyField(blank=True, null=True, to='ovp_projects.JobDate'),
        ),
        migrations.AddField(
            model_name='job',
            name='project',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='ovp_projects.Project'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-11-15 17:45
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_projects', '0011_auto_20161115_1744'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Role',
            new_name='VolunteerRole',
        ),
    ]



from rest_framework import serializers

from ovp_projects.models import VolunteerRole

class VolunteerRoleSerializer(serializers.ModelSerializer):
  class Meta:
    model = VolunteerRole
    fields = ['name', 'prerequisites', 'details', 'vacancies']



XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX



from django.db import models
from django.utils.translation import ugettext_lazy as _

class JobDate(models.Model):
  name = models.CharField(_("Start date"), blank=True, null=True, max_length=20)
  start_date = models.DateTimeField(_("Start date"))
  end_date = models.DateTimeField(_("End date"))

  class Meta:
    app_label = 'ovp_projects'
    verbose_name = _('job date')
    verbose_name_plural = _('job dates')


class Job(models.Model):
  project = models.OneToOneField('Project', blank=True, null=True)
  start_date = models.DateTimeField(_("Start date"), blank=True, null=True)
  end_date = models.DateTimeField(_("End date"), blank=True, null=True)
  can_be_done_remotely = models.BooleanField(_("This job can be done remotely"), default=False)
  dates = models.ManyToManyField('JobDate', blank=True)

  def __str__(self):
    return "{} - {}".format(self.start_date, self.end_date)

  def update_dates(self):
    start = self.dates.all().order_by('start_date').first().start_date
    end   = self.dates.all().order_by('-end_date').first().end_date
    self.start_date = start
    self.end_date = end
    self.save()

  class Meta:
    app_label = 'ovp_projects'
    verbose_name = _('job')
    verbose_name_plural = _('jobs')



from ovp_projects import models
from ovp_projects.serializers.job import JobSerializer
from ovp_projects.serializers.work import WorkSerializer

from rest_framework import serializers

from collections import OrderedDict
from functools import wraps


"""
Validators
"""
def disponibility_validate(disponibility):
  if disponibility['type'] not in ['work', 'job']:
    raise serializers.ValidationError({'type': ["Must have either be 'work' or 'job'."]})
  else:
    disp_type = disponibility['type']

  disponibility_object_validate(disponibility, disp_type)


def disponibility_object_validate(d, k):
  if k not in d:
    raise serializers.ValidationError({k: ["This field is required if type=\"{}\".".format(k)]})

  if k == 'work':
    serializer = WorkSerializer(data=d[k])
  if k == 'job':
    serializer = JobSerializer(data=d[k])
  serializer.is_valid(raise_exception=True)



"""
Representation decorator
"""
def add_disponibility_representation(func):
  @wraps(func)
  def _impl(self, instance):
    # We pop disponibility field to avoid AttributeError on default Serializer.to_representation
    for i, field in enumerate(self._readable_fields):
      if field.field_name == "disponibility":
        disponibility = self._readable_fields.pop(i)

    ret = func(self, instance)
    self._readable_fields.insert(i, disponibility) # Put disp back

    # Add disponibility representation
    obj = None
    ret["disponibility"] = None

    try:
      type = "job"
      obj = JobSerializer().to_representation(instance.job)
    except models.Job.DoesNotExist:
      try:
        type = "work"
        obj = WorkSerializer().to_representation(instance.work)
      except models.Work.DoesNotExist: # pragma: no cover
        pass

    if obj:
      ret["disponibility"] = {
        "type": type,
        type: obj
      }


    return ret
  return _impl


"""
Serializer
"""
class DisponibilitySerializer(serializers.Serializer):
  type = serializers.CharField(max_length=4)
  work = WorkSerializer(required=False)
  job = JobSerializer(required=False)

  class Meta:
    validators=[disponibility_validate]



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone

from ovp_projects import emails

class Apply(models.Model):
  user = models.ForeignKey('ovp_users.User', blank=True, null=True)
  project = models.ForeignKey('ovp_projects.Project')
  status = models.CharField(_('name'), max_length=30)
  date = models.DateTimeField(auto_now_add=True, blank=True) # created date
  canceled = models.BooleanField(_("Canceled"), default=False)
  canceled_date = models.DateTimeField(_("Canceled date"), blank=True, null=True)
  email = models.CharField(_('Email'), max_length=200, blank=True, null=True)

  def mailing(self, async_mail=None):
    return emails.ApplyMail(self, async_mail)

  def save(self, *args, **kwargs):
    if self.canceled:
      self.canceled_date = timezone.now()
      self.status = 'unapplied'
      self.mailing().sendUnappliedToVolunteer({'apply': self})
      self.mailing().sendUnappliedToOwner({'apply': self})
    else:
      self.canceled_date = None
      self.status = 'applied'
      self.mailing().sendAppliedToVolunteer({'apply': self})
      self.mailing().sendAppliedToOwner({'apply': self})

    return_data = super(Apply, self).save(*args, **kwargs)

    # Updating project applied_count
    # get_volunteers_numbers return a function, so ()()
    self.project.applied_count = self.project.get_volunteers_numbers()()
    self.project.save()

    return return_data


  class Meta:
    app_label = 'ovp_projects'
    verbose_name = _('apply')
    verbose_name_plural = _('applies')
    unique_together = (("email", "project"), )




from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify

from ovp_projects.models.apply import Apply
from ovp_projects import emails


class Project(models.Model):
  """
  Project model
  """
  image = models.ForeignKey('ovp_uploads.UploadedImage', blank=False, null=True)
  address = models.OneToOneField('ovp_core.GoogleAddress', blank=True, null=True)
  skills = models.ManyToManyField('ovp_core.Skill')
  causes = models.ManyToManyField('ovp_core.Cause')

  # Relationships
  owner = models.ForeignKey('ovp_users.User')
  organization = models.ForeignKey('ovp_organizations.Organization', blank=False, null=True)
  roles = models.ManyToManyField('VolunteerRole', verbose_name=_("Volunteer Roles"), blank=True)

  # Fields
  name = models.CharField(_('Project name'), max_length=100)
  slug = models.SlugField(max_length=100, unique=True, blank=True, null=True)
  published = models.BooleanField(_("Published"), default=False)
  highlighted = models.BooleanField(_("Highlighted"), default=False, blank=False)
  applied_count = models.IntegerField(blank=False, null=False, default=0)

  # Date fields
  published_date = models.DateTimeField(_("Published date"), blank=True, null=True)
  closed = models.BooleanField(_("Closed"), default=False)
  closed_date = models.DateTimeField(_("Closed date"), blank=True, null=True)
  deleted = models.BooleanField(_("Deleted"), default=False)
  deleted_date = models.DateTimeField(_("Deleted date"), blank=True, null=True)
  created_date = models.DateTimeField(auto_now_add=True)
  modified_date = models.DateTimeField(auto_now=True)

  # About
  details = models.TextField(_('Details'), max_length=3000)
  description = models.TextField(_('Short description'), max_length=160, blank=True, null=True)

  def mailing(self, async_mail=None):
    return emails.ProjectMail(self, async_mail)

  '''
  Data methods
  '''
  def get_phone(self):
    return self.owner.phone

  def get_email(self):
    return self.owner.email

  def get_volunteers_numbers(self):
    return Apply.objects.filter(project=self, canceled=False).count

  '''
  Model operation methods
  '''
  def delete(self, *args, **kwargs):
      self.deleted = True
      self.published = False
      self.save()

  def save(self, *args, **kwargs):
    if self.pk is not None:
      orig = Project.objects.get(pk=self.pk)
      if not orig.published and self.published:
        self.published_date = timezone.now()
        self.mailing().sendProjectPublished({'project': self})

      if not orig.closed and self.closed:
        self.closed_date = timezone.now()
        self.mailing().sendProjectClosed({'project': self})

      if not orig.deleted and self.deleted:
        self.deleted_date = timezone.now()
    else:
      # Project being created
      self.slug = self.generate_slug()
      self.mailing().sendProjectCreated({'project': self})

    # If there is no description, take 100 chars from the details
    if not self.description and len(self.details) > 100:
      self.description = self.details[0:100]

    self.modified_date = timezone.now()

    return super(Project, self).save(*args, **kwargs)

  def generate_slug(self):
    if self.name:
      slug = slugify(self.name)[0:99]
      append = ''
      i = 0

      query = Project.objects.filter(slug=slug + append)
      while query.count() > 0:
        i += 1
        append = '-' + str(i)
        query = Project.objects.filter(slug=slug + append)
      return slug + append
    return None

  def __str__(self):
      return  '%s' % (self.name)

  class Meta:
    app_label = 'ovp_projects'
    verbose_name = _('project')
    verbose_name_plural = _('projects')


class VolunteerRole(models.Model):
  """
  Volunteer role model
  """
  name = models.CharField(_('Role name'), max_length=50, blank=True, null=True, default=None)
  prerequisites = models.TextField(_('Prerequisites'), max_length=1024, blank=True, null=True, default=None)
  details = models.TextField(_('Details'), max_length=1024, blank=True, null=True, default=None)
  vacancies = models.PositiveSmallIntegerField(_('Vacancies'), blank=True, null=True, default=None)

  class Meta:
    app_label = 'ovp_projects'
    verbose_name = _('volunteer role')
    verbose_name_plural = _('volunteer roles')

  def __str__(self):
    return  '%s - %s - %s (%s vacancies)' % (self.name, self.details, self.prerequisites, self.vacancies)



from django.db import models
from django.utils.translation import ugettext_lazy as _


class Work(models.Model):
  project = models.OneToOneField('Project', blank=True, null=True)
  weekly_hours = models.PositiveSmallIntegerField(_('Weekly hours'), blank=True, null=True)
  description = models.CharField(_('Description'), max_length=4000)
  can_be_done_remotely = models.BooleanField(_("This job can be done remotely"), default=False)

  def __str__(self):
    return "%s hours per week" % (self.weekly_hours)

  class Meta:
    app_label = 'ovp_projects'
    verbose_name = _('work')
    verbose_name_plural = _('works')



from django.conf.urls import url, include
from rest_framework import routers

from ovp_projects import views

router = routers.DefaultRouter()
router.register(r'projects', views.ProjectResourceViewSet, 'project')

urlpatterns = [
  url(r'^', include(router.urls)),
]
